//------------------------------------------------------------------------
//
// Author      : Maarten "McCLaw" Kronberger
// Email       : sulacomcclaw@hotmail.com
// Website     : http://www.sulaco.co.za
// Date        : 13 February 2004
// Version     : 1.0
// Description : This is the basic application template that I use
//				 to create my OpenGL applications in Delphi.net
//				 To use this you will need to install the Tao assemblies.
//
//				 These assemblies as well as instructions on how to use and
//				 install them can be found here : http://csgl.sourceforge.net/
//
//				 If you have any queries or bug reports, please mail me.
//
//------------------------------------------------------------------------
unit frmDisplay;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Tao.OpenGl,
  Tao.Platform.Windows,
  System.ComponentModel,
  Borland.Vcl.ExtCtrls;

type
  TForm1 = class(TForm)
    pnlRender: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure glInit();
    procedure glDraw();
    procedure pnlRenderResize(Sender: TObject);
    procedure Idle(Sender: TObject; var Done: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    rc : HGLRC;    // Rendering Context
    dc  : HDC;     // Device Context
    ElapsedTime, AppStart, LastTime : DWord;  // Timing variables
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.nfm}

{------------------------------------------------------------------}
{  Create the form and initialise openGL                           }
{------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
var pfd : TPIXELFORMATDESCRIPTOR;
    pf  : Integer;
begin
  // OpenGL initialise
  dc:=GetDC(pnlRender.Handle);

  // PixelFormat
  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
  pfd.iPixelType:=PFD_TYPE_RGBA;      // PFD_TYPE_RGBA or PFD_TYPEINDEX
  pfd.cColorBits:=32;

  pf :=ChoosePixelFormat(dc, pfd);   // Returns format that most closely matches above pixel format
  SetPixelFormat(dc, pf, pfd);

  rc :=wglCreateContext(dc);    // Rendering Context = window-glCreateContext
  wglMakeCurrent(dc,rc);        // Make the DC (Form1) the rendering Context

  // Initialise GL environment variables
  glInit;
  pnlRenderResize(sender);    // sets up the perspective
  AppStart :=GetTickCount();

  // when the app has spare time, render the GL scene
  Application.OnIdle := Idle;
end;

{------------------------------------------------------------------}
{  Initialise OpenGL                                               }
{------------------------------------------------------------------}
procedure TForm1.glInit();
begin
  GL.glClearColor(0.0, 0.0, 0.0, 0.0); 	   // Black Background
  GL.glShadeModel(GL.GL_SMOOTH);                 // Enables Smooth Color Shading
  GL.glClearDepth(1.0);                       // Depth Buffer Setup
  GL.glEnable(GL.GL_DEPTH_TEST);                 // Enable Depth Buffer
  GL.glDepthFunc(GL.GL_LESS);		           // The Type Of Depth Test To Do

  GL.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);   //Realy Nice perspective calculations
end;

{------------------------------------------------------------------}
{  If the panel resizes, reset the GL scene                        }
{------------------------------------------------------------------}
procedure TForm1.pnlRenderResize(Sender: TObject);
begin
  GL.glViewport(0, 0, pnlRender.Width, pnlRender.Height);    // Set the viewport for the OpenGL window
  GL.glMatrixMode(GL.GL_PROJECTION);        // Change Matrix Mode to Projection
  GL.glLoadIdentity();                   // Reset View
  Glu.gluPerspective(45.0, pnlRender.Width/pnlRender.Height, 1.0, 500.0);  // Do the perspective calculations. Last value = max clipping depth

  GL.glMatrixMode(GL.GL_MODELVIEW);         // Return to the modelview matrix
end;

{------------------------------------------------------------------}
{  Application onIdle event                                        }
{------------------------------------------------------------------}
procedure TForm1.Idle(Sender: TObject; var Done: Boolean);
begin
  Done := FALSE;

  LastTime :=ElapsedTime;
  ElapsedTime :=GetTickCount() - AppStart;      // Calculate Elapsed Time
  ElapsedTime :=(LastTime + ElapsedTime) DIV 2; // Average it out for smoother movement

  glDraw();                         // Draw the scene
  SwapBuffers(DC);                  // Display the scene
end;

{------------------------------------------------------------------}
{  Function to draw the actual scene                               }
{------------------------------------------------------------------}
procedure TForm1.glDraw();
begin
  GL.glClear(GL.GL_COLOR_BUFFER_BIT or GL.GL_DEPTH_BUFFER_BIT);    // Clear The Screen And The Depth Buffer
  GL.glLoadIdentity();                                       // Reset The View

  GL.glTranslatef(0, 0, -4);

  GL.glRotatef(ElapsedTime/10, 0, 1, 0);

  GL.glBegin(GL.GL_TRIANGLES);
    GL.glColor3f(1, 0, 0);  GL.glVertex3f(-1, -1, 0);
    GL.glColor3f(0, 1, 0);  GL.glVertex3f( 1, -1, 0);
    GL.glColor3f(0, 0, 1);  GL.glVertex3f( 0,  1, 0);
  GL.glEnd();
end;

{------------------------------------------------------------------}
{  Release rendering context when form gets detroyed               }
{------------------------------------------------------------------}
procedure TForm1.FormDestroy(Sender: TObject);
begin
  wglMakeCurrent(0,0);
  wglDeleteContext(rc);
end;

{------------------------------------------------------------------}
{  Monitors all keypress events for the app                        }
{------------------------------------------------------------------}
procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then
    Close;
end;

end.
